
; unpaged RAM based monitor for the Micronic 1000

; Display attribute equates

DispOn:	EQU	20h			; display on bit
DispBlnk:	EQU	08h			; cursor blink bit
DispCrsr:	EQU	04h			; cursor on bit
DispGrap:	EQU	02h			; display graph mode bit
DispExCg:	EQU	01h			; external GC bit

; equates, others to be added as needed

KBD_R:	EQU	00h			; key matrix read port
KBD_W:	EQU	02h			; key matrix write port
LCD_D:	EQU	03h			; LCD data port
Port_04:	EQU	04h
Port_05:	EQU	05h
RTC_A:	EQU	08h			; RTC address port
LCD_C:	EQU	23h			; LCD command port
RTC_D:	EQU	28h			; RTC data port
Port_2A:	EQU	2Ah			;
Port_2B:	EQU	2Bh			; .... xxxx = beep tone
						;  0001=high, 1111=low, 0000=off
Port_2C:	EQU	2Ch			; ...x .... = backlight on/off 1/0
Port_2D:	EQU	2Dh			;
Port_46:	EQU	46h			; LCD contrast port
MEM_P:	EQU	47h			; memory page
Port_4B:	EQU	4Bh			; end IR port input

LAB_F81A	EQU	F81Ah			; Micronic's stack base

	ORG	8000h				; moved it to RAM
LAB_F5F3
	RETI					; IRQ_vect default - IRQ routine
						; (usually RETI or JP xxxxh)
	DB	00h				; dummy for jump address
	RETN					; NMI_vect default - NMI routine
						; (usually RETN or JP xxxxh)
	DB	00h				; dummy for jump address
Start:
	DI					; disable interrupts
	LD	SP,LAB_F81A			; set the stack
	IM	1				; set interrupt mode 1

;**********************************************************

	LD	A,#08h			; set tone
	OUT	(Port_2B),A			; tone on

	LD	A,#20h			; dunno, but it's in ..
	OUT	(Port_2A),A			; .. the original code

	LD	BC,0FA0h			; short delay
LAB_0159
	NOP					; loop pad byte
	DEC	BC				; count down
	LD	A,B				; get high byte
	OR	C				; or in low byte
	JR	NZ,LAB_0159			; loop if not zero

	IN	A,(Port_05)			; ..
	LD	A,#FFh			; ..
	OUT	(Port_04),A			; .. (doesn't help much though!)
	XOR	A				; clear A
	OUT	(Port_2B),A			; tone off

;**********************************************************

	LD	HL,Set_def			; point to default variables table
	LD	DE,key_ptr			; point to variables start
	LD	BC,LCD_def-Set_def	; table length
	LDIR					; copy table

	LD	HL,LCD_def			; point to default LCD table
	LD	DE,TabSize			; point to LCD table start
	LD	BC,Defend-LCD_def		; table length
	LDIR					; copy table
Warm:
	CALL	LCD_Cont			; set contrast port
	CALL	LCD_init			; set initial values

	LD	A,#08h			; display on
	CALL	LCD_setchrmode		; set display mode to chr
	LD	A,(DispMode)		; get display mode
	OR	#DispBlnk+DispCrsr	; turn the cursor & blink on
	LD	(DispMode),A		; save mode
	CALL	LCD_mode			; set display mode
	CALL	LCD_clr			; clear the display

	LD	HL,Tst1			; point to message
	LD	BC,Tst1End-Tst1		; mesage length	
	CALL	WriteL			; write the message
	CALL	LAB_2828			; test clock
	OR	A				; set the flags
	DI					; disable interrupts
	IN	A,(Port_05)			; clear interrupt flag byte
	LD	A,#FFh			; turn everything off
	OUT	(Port_04),A			; disable all interrupts
	JR	NZ,FailedM			; if failed do message

	LD	HL,MPassed			; point to message
	LD	BC,MPaend-MPassed		; mesage length
	JR	Result			; go do it

FailedM:
	LD	HL,MFailed			; point to message
	LD	BC,MFaend-MFailed		; mesage length
Result:
	CALL	WriteL			; write the message
	LD	HL,Tst2			; point to message
	LD	BC,Tst2End-Tst2		; mesage length	
	CALL	WriteL			; write the message
	CALL	Wait_key			; wait until key pressed
	CALL	LCD_clr			; clear the display

; display a startup message on the LCD

	LD	HL,Msg			; point to message
	LD	BC,MsgEnd-Msg		; mesage length	
	CALL	WriteL			; write the message
	CALL	Wait_key			; wait until key pressed

; re-enter main loop, clearing the display.

main_clr:
	CALL	LCD_clr			; clear the display
	LD	HL,Mon			; point to message
	LD	BC,MonEnd-Mon		; mesage length	
	CALL	WriteL			; write the message

; main monitor loop

main_a:
	LD	BC,00A8h			; line 6, chr 9
	CALL	Set_cursor			; set the cursor
	LD	HL,(mon_addr)		; get the monitor address
	CALL	Test_byte			; test byte at (HL) Result in AF
	LD	HL,RamM			; point to message
	LD	BC,RamEnd-RamM		; mesage length	
	JR	Z,main_ram			; jump if RAM

	LD	HL,NRamM			; point to message
main_ram:
	CALL	WriteL			; write the message
	CALL	Disp_page			; show page (if any)
	CALL	Disp_addr			; display the system address
main_d:
	CALL	Disp_data			; display the system data
	LD	A,(mon_mode)		; get the monitor mode
	OR	A				; set the flags
	JR	NZ,Not_addr			; jump if data mode

						; sets the cursor back if in address mode
	LD	BC,0051h			; line 3, chr 18
	CALL	Set_cursor			; set the cursor
Not_addr:
	CALL	Wait_key			; wait until new key pressed
	CP	#0Dh				; is it enter?
	JP	Z,K_enter			; jump if so

	CP	#14h				; was it END (address -1)
	JP	Z,K_end			; jump if so

	CP	#'I'				; is it insert? (address mode)
	JR	NZ,K_not_ins		; jump if not

						; key was insert
	LD	A,(mon_mode)		; get the monitor mode
	CP	#02h				; compare with page mode
	JR	Z,Not_addr			; jump if page mode

	XOR	A				; clear A
	LD	(mon_mode),A		; save it
	JR	main_a			; back to monitor loop

K_not_ins:
	CP	#'N'				; is it N/Z key?
	JP	Z,Start			; jump if so (cold start)

	CP	#'M'				; is it 'M'?
	JP	Z,Warm			; jump if so (warm start)

	CP	#'J'				; is it J/Y key?
	JP	Z,PortMon			; jump if so (port monitor)

	CP	#'K'				; is it K key?
	JP	Z,MemMon			; jump if so (mem monitor)

	CP	#'H'				; is it 'H'? (help)
	JR	NZ,K_not_help		; jump if not

	CALL	LCD_clr			; clear the display
	LD	HL,HelpM			; point to message
	LD	BC,HelpEnd-HelpM		; mesage length	
	CALL	WriteL			; write the message
	CALL	Wait_key			; wait until key pressed
	CALL	LCD_clr			; clear the display

	LD	A,(mon_mode)		; get the monitor mode
	CP	#02h				; compare with page mode
	JR	Z,K_page			; jump if page mode

	JP	main_clr			; back to monitor loop

K_not_help:
	CP	#'L'				; is it 'L'? (set mem bank)
	JR	Z,K_page			; jump if was

	CP	#'G'				; is it 'G'? (go from address)
	JR	NZ,K_not_go			; jump if not

	LD	HL,(mon_addr)		; point to monitor address
	JP	(HL)				; jump to it

K_not_go:
	JR	NC,N_ram			; not hex key so go do monitor loop

	CP	#'0'				; compare with '0'
	JR	C,N_ram			; not hex key so go do monitor loop

;	JP	Hex_key			; go do hex key

; key was hex key

Hex_key:

	SUB	#30h				; makes '0'-'9' equal 0-9
	CP	#0Ah				; is it 0-9 ?
	JP	C,was_hex			; if so then go do it

	CP	#11h				; is it inbetween 9 and A ?
	JR	C,Hex_exit			; if so then exit
	SUB	#07h				; else convert alpha
	CP	#10h				; make sure it's in range
	jp	C,was_hex			; continue if so

Hex_exit:
	LD	A,(mon_mode)		; get the monitor mode
	CP	#02h				; compare with page mode
	JR	Z,Not_addr			; is page mode return

	JP	main_a			; back to monitor loop

was_hex:
	LD	C,A				; save key for now
	LD	A,(mon_mode)		; get the monitor mode
	OR	A				; set the flags
	JR	Z,Hex_addr			; jump if addr mode

	DEC	A				; -1
	JR	NZ,Hex_page			; is page mode

						; hex key in data mode
Hex_data:
	LD	HL,(mon_addr)		; address pointer in HL
	CALL	Test_byte			; test byte at (HL) Result in AF
	JR	NZ,N_ram			; exit if not RAM

	LD	A,C				; get key back
	RLD					; rotate nibble into byte
	CALL	Disp_data			; display data
N_ram:
	JP	main_a			; back to monitor loop

; hex key in address mode

Hex_addr:
	LD	HL,mon_addr			; get address pointer
	LD	A,C				; get key back
	RLD					; rotate nibble into low byte
	INC	HL				; point to high byte
	RLD					; rotate nibble into high byte
	CALL	Disp_addr			; dsiplay addr
	JP	main_a			; back to monitor loop

; hex key in page mode

Hex_page:
	LD	HL,bank_b			; address pointer in HL
	LD	A,C				; get key back
	RLD					; rotate nibble into byte
	JR	Page_new			; dispaly & back to monitor loop

; start of the subroutines ..

; key was page

K_page:
	CALL	LCD_clr			; clear the display
	LD	HL,PageM			; point to message
	LD	BC,PageEnd-PageM		; mesage length	
	CALL	WriteL			; write the message
Set_page:
	LD	BC,0051h			; line 3, chr 18
	CALL	Set_cursor			; set the cursor
	LD	A,(bank_p)			; get current page
	LD	(bank_b),A			; save to buffer
	CALL	Disp_A			; display it
Page_new:
	LD	BC,0091h			; line 5, chr 18
	CALL	Set_cursor			; set the cursor
	LD	A,(bank_b)			; get new page
	CALL	Disp_A			; display it

	LD	A,#02h			; set page mode
	LD	(mon_mode),A		; save monitor mode
	LD	BC,0091h			; line 5, chr 18
	CALL	Set_cursor			; set the cursor
	JP	Not_addr			; back to monitor loop

; key was enter

K_enter:
	LD	A,(mon_mode)		; get the monitor mode
	OR	A				; set the flags
	JR	Z,is_addr			; jump if addr mode (go enter data mode)

	DEC	A				; -1
	JR	NZ,is_page			; is page mode (set page & return to monitor)

	LD	HL,(mon_addr)		; get address
	INC	HL				; increment it
	LD	(mon_addr),HL		; save it
	JP	main_a			; back to monitor loop

is_addr:
	INC	A				; make it data mode
Set_mode:
	LD	(mon_mode),A		; save it back
	JP	main_d			; back to monitor loop

is_page:
	LD	A,(bank_b)			; get new bank no.
	LD	(bank_p),A			; save as new bank no.
	OUT	(MEM_P),A			; set memory page
	JP	Set_page			; go display new page


; key was end

K_end:
	LD	A,(mon_mode)		; get the monitor mode
	CP	#02h				; compare with page mode
	JR	NZ,Dec_addr			; jump if not page mode

	XOR	A				; clear A
	LD	(mon_mode),A		; save it back
	JP	main_clr			; back to monitor loop

						; was either address or data mode
Dec_addr
	LD	HL,(mon_addr)		; get address
	DEC	HL				; decrement it
	LD	(mon_addr),HL		; save it
	JP	main_a			; back to monitor loop

; monitor port addresses. display a table of 24 port reads.

PortMon:
	CALL	LCD_clr			; clear the display
	LD	BC,0004h			; line 0, chr 5
	CALL	Set_cursor			; set the cursor
	CALL	prt_str			; print the title string
	DB	'Port monitor',00h	; title
	LD	HL,Key_hex			; get hex key table
	LD	(key_ptr),HL		; set it a current
port_dl:
	CALL	port_prt			; display the port contents
	CALL	Wait_key			; wait until new key pressed
	CP	#1Dh				; ENTER key for this mode
	JR	Z,exit_port			; quit if ENTER

	CP	#16h				; YES key for this mode
	JR	NZ,not_yes			; jump if not YES

	LD	A,(port_adr)		; get port addresses start
	ADD	A,#04h			; increment address 4 ports
	LD	(port_adr),A		; save new port addresses start
	JR	port_dl			; go refresh display

not_yes:
	CP	#11h				; NO key for this mode
	JR	NZ,port_dl			; jump if not NO (refresh display)

	LD	A,(port_adr)		; get port addresses start
	SUB	#04h				; decrement address 4 ports
	LD	(port_adr),A		; save new port addresses start
	JR	port_dl			; go refresh display

exit_port:
	LD	HL,Key_tab			; get original key table
	LD	(key_ptr),HL		; re set it
	JP	main_clr			; go back to main loop clearing the display

; write port contents. destroys A

port_prt:
	PUSH	BC				; save BC
	PUSH	DE				; save DE
	PUSH	HL				; save HL

	LD	BC,0020h			; line 2, chr 0
	CALL	Set_cursor			; set the cursor
	LD	A,(port_adr)		; get port addresses start
	LD	C,A				; copy it

	LD	E,#06h			; lines to do
port_line:
	CALL	prt_str			; print line leader string
	DB	'Prt (',00h			; leader
	LD	A,C				; get address
	CALL	Byte_out			; display address
	LD	A,#')'			; close bracket
	CALL	LCD_print			; out to display
	LD	B,#04h			; byte count
port_loop:
	LD	A,#' '			; space before byte
	CALL	LCD_print			; out to display
	IN	A,(C)				; get port contents
	INC	C				; point to next port
	CALL	Byte_out			; display byte
	DJNZ	port_loop			; loop until all four done

	CALL	prt_str			; print line trailer string
	DB	0Dh,0Ah,00h			; trailer
	DEC	E				; decrement line count
	JR	NZ,port_line		; loop until all lines done

	POP	HL				; restore HL
	POP	DE				; restore DE
	POP	BC				; restore BC

	RET

; block dump. display a table of 24 addresses.

MemMon:
	CALL	LCD_clr			; clear the display
	LD	BC,0004h			; line 0, chr 5
	CALL	Set_cursor			; set the cursor
	CALL	prt_str			; print the title string
	DB	'Memory dump',00h		; title
	LD	HL,Key_hex			; get hex key table
	LD	(key_ptr),HL		; set it a current
mem_dl:
	CALL	mem_prt			; display the memory contents
	CALL	Wait_key			; wait until new key pressed
	CP	#1Dh				; ENTER key for this mode
	JR	Z,exit_mem			; quit if ENTER

	CP	#16h				; YES key for this mode
	JR	NZ,mnot_yes			; jump if not YES

	LD	HL,(mon_addr)		; get mem addresses start
	LD	A,L				; copy low byte
	AND	#FCh				; clear bottom two bits
	ADD	A,#04h			; increment address 4 bytes
	LD	L,A				; save it back
	LD	A,#00h			; clear A
	ADC	A,H				; add high byte
	LD	H,A				; copy it back
	LD	(mon_addr),HL		; save new memory addresses start
	JR	mem_dl			; go refresh display

mnot_yes:
	CP	#11h				; NO key for this mode
	JR	NZ,mem_dl			; jump if not NO (refresh display)

	LD	HL,(mon_addr)		; get mem addresses start
	LD	A,L				; copy low byte
	AND	#FCh				; clear bottom two bits
	SUB	#04h				; decrement address 4 bytes
	LD	L,A				; save it back
	LD	A,H				; get H
	SBC	A,#00h			; subtract carry from high byte
	LD	H,A				; copy it back
	LD	(mon_addr),HL		; save new memory addresses start
	JR	mem_dl			; go refresh display

exit_mem:
	LD	HL,Key_tab			; get original key table
	LD	(key_ptr),HL		; re set it
	JP	main_clr			; go back to main loop clearing the display


; write memory contents. destroys A

mem_prt:
	PUSH	BC				; save BC
	PUSH	DE				; save DE
	PUSH	HL				; save HL

	LD	BC,0020h			; line 1, chr 1
	CALL	Set_cursor			; set the cursor
	LD	HL,(mon_addr)		; get memory addresses start
	LD	A,L				; get low byte
	AND	#FCh				; clear bottom two bits
	LD	L,A				; copy back

	LD	C,#06h			; lines to do
mem_line:
	CALL	Word_out			; display address
	LD	B,#04h			; byte count
mem_hex_loop:
	LD	A,#' '			; space before byte
	CALL	LCD_print			; out to display
	LD	A,(HL)			; get memory contents
	INC	HL				; point to next address
	CALL	Byte_out			; display byte
	DJNZ	mem_hex_loop		; loop until all four done

	LD	A,#' '			; space before ascii
	CALL	LCD_print			; out to display
	LD	DE,0004h			; set offset
	OR	A				; clear carry
	SBC	HL,DE				; set pointer back
	LD	B,#04h			; byte count
mem_asci_loop:
	LD	A,(HL)			; get memory contents
	CP	#20h				; compare with space
	JR	C,no_print			; jump if unprintable

	CP	#80h				; compare with max printable+1
	JR	C,yes_print			; is printable

no_print:
	LD	A,#'.'			; else make  dot
yes_print:
	INC	HL				; point to next address
	CALL	LCD_print			; out to display
	DJNZ	mem_asci_loop		; loop until all four done

	CALL	prt_str			; print line trailer string
	DB	0Dh,0Ah,00h			; trailer
	DEC	C				; decrement line count
	JR	NZ,mem_line			; loop until all lines done

	CALL	prt_str			; print screen trailer string
	DB	'Yes + No - Enter >>'	; trailer
	DB	00h				;
	POP	HL				; restore HL
	POP	DE				; restore DE
	POP	BC				; restore BC

	RET

; string out. string to be sent is immediately after the CALL and must end in 00h
; destroys no registers, returns nothing

prt_str:
	EX	(SP),HL			; get pointer to string
	PUSH	AF				; save A
str_nxt:
	LD	A,(HL)			; get byte
	INC	HL				; point to next
	OR	A				; set Zero flag
	JR	Z,str_end			; finished
	CALL	LCD_print			; out to LCD
	JR	str_nxt			; go get next character
str_end:
	POP	AF				; restore A
	EX	(SP),HL			; put return address on stack
	RET					;

; test byte at (HL) returns Z = true if RAM at address. leaves (HL) unchanged.

Test_byte:
	LD	A,(HL)			; read (HL) into A
	CPL					; complement it
	LD	(HL),A			; write it back
	LD	A,(HL)			; re-read it
	CPL					; complement it again (back to original)
	LD	(HL),A			; write it back
	CP	(HL)				; compare bytes
	RET					;

; display address as hex word. leaves cursor at end of address.

Disp_addr:
	LD	BC,004Eh			; line 3, chr 15
	CALL	Set_cursor			; set the cursor
Disp_ad2:
	LD	HL,(mon_addr)		; get the monitor address
Disp_HL:
	CALL	Word_out			; display it
	LD	A,#08h			; backspace
	CALL	LCD_print			; out to display
	RET

; display data as hex byte. leaves cursor at end of data.

Disp_data:
	LD	BC,006Eh			; line 4, chr 15
	CALL	Set_cursor			; set the cursor
Disp_dat2:
	LD	A,(HL)			; get the monitor byte
Disp_A:
	CALL	Byte_out			; display it
	LD	A,#08h			; backspace
Disp_E:
	CALL	LCD_print			; out to display
	RET

; display page as hex byte.

Disp_page:
	LD	BC,002Eh			; line 2, chr 15
	CALL	Set_cursor			; set the cursor
	LD	A,(mon_addr+1)		; get the monitor address high byte
	CP	#80h				; compare with unpaged boundary
	LD	A,(bank_p)			; get the current page
	JR	C,Disp_A			; is paged mem so go show it

						; else was unpaged mem so show '**'
	LD	A,#'*'			; hash
	CALL	LCD_print			; out to display
	LD	A,#'*'			; hash
	JP	Disp_E			; dispaly & exit

; output A as hex pair. destroys A

Byte_out:
	PUSH	AF				; save A
	RRCA					; rotate top nibble ...
	RRCA					; ... into lower nibble
	RRCA					; ..
	RRCA					; ..
	CALL	Hex2ascii			; convert lower nibble A to ASCII hex
	CALL	LCD_print			; out to LCD
	POP	AF				; restore A
	CALL	Hex2ascii			; convert lower nibble A to ASCII hex
	CALL	LCD_print			; out to LCD
	RET					;

; output HL as hex word. destroys A

Word_out:
	PUSH	HL				; save HL
	PUSH	HL				; save HL
	LD	A,H				; copy H to A
	CALL	Byte_out			; output A as hex pair
	POP	HL				; restore HL
	LD	A,L				; copy L to A
	CALL	Byte_out			; output A as hex pair
	POP	HL				; restore HL
	RET					;

; convert lower nibble A to ASCII hex

Hex2ascii:
	AND	#0Fh				; clear high nibble
	ADD	A,#90h			; ..
	DAA					; ..
	ADC	A,#40h			; ..
	DAA					; ..
	RET					;

; Writes the message at (HL) of length BC to the display

WriteL:
	LD	A,(HL)			; get byte
	CALL	LCD_print			; out to LCD
	INC	HL				; point to next chr
	DEC	BC				; decrement count
	LD	A,B				; get high byte
	OR	C				; OR in low byte
	JR	NZ,WriteL			; loop if not all done
	RET					; done

; sets the LCD contrast

LCD_Cont:
	LD	A,(DispCon)			; get display contrast byte
	OUT	(Port_46),A			; out to contrast port
	RET

; initialise 120x64 LCD display. aets time divisions to 64. also sets cursor
; size to line 7 (underline) only.

LCD_init:
	LD	A,#03h			; set time divisions instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,#3Fh			; set to 64
	OUT	(LCD_D),A			; set mode
	LD	A,#04h			; set cursor size instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,#07h			; set to 7
	OUT	(LCD_D),A			; set size
	RET

; output instruction. Unknown if I can read the busy flag so just wait a short
; time before (& after) attempting to write a new instruction.

LCD_instr:
	PUSH	HL				; short wait ..
	POP	HL				; ..
	OUT	(LCD_C),A			; out to display
	PUSH	HL				; short wait ..
	POP	HL				; ..
	RET					; done

; turn on the LCD without changing the mode parameters.

LCD_on:
	LD	A,(DispMode)		; get the display mode
	OR	#DispOn			; turn on bit
	LD	(DispMode),A		; save mode
	JP	LCD_mode			; set display mode & return

; turn off the LCD without changing the mode parameters.

LCD_off:
	LD	A,(DispMode)		; get the display mode
	AND	#FFh-DispOn			; turn off bit
	LD	(DispMode),A		; save mode

; set the LCD mode. mode byte should be in A

LCD_mode:
	PUSH	AF				; save mode byte
	XOR	A				; clear A
	CALL	LCD_instr			; set display control instruction
	POP	AF				; get mode byte back
	OUT	(LCD_D),A			; set mode
	RET

; Clear LCD display. Turns off display, checks mode and then fills 2048
; display bytes with either 00h (graph mode) or 20h (chr mode). It also
; homes the cursor.

; If the commented out instructions are restored then the display will be
; blanked during clearing.

LCD_clr:
	LD	A,(DispMode)		; get mode
;	AND	#1Fh				; display off
;	CALL	LCD_mode			; set display mode
	AND	#02h				; check chr/graph mode
	LD	A,#00h			; clear A (don't set flags!)
	LD	HL,CursorAt			; get pointer to cursor pos'n
	LD	(HL),A			; clear byte
	INC	HL				; next byte
	LD	(HL),A			; clear byte
	JR	NZ,ClGraph			; go clear graph mode
	LD	A,#20h			; space chr to clear chr mode display
ClGraph:
	PUSH	AF				; save it
	LD	BC,0000h			; reset cursor
	CALL	Set_cursor			; set cursor position
	LD	C,#08h			; 8 x 256 = 2048 bytes
	LD	A,#0Ch			; write display data
	CALL	LCD_instr			; set instruction
	POP	AF				; get byte back
Clear_l:
	OUT	(LCD_D),A			; chr out to display
	DJNZ	Clear_l			; loop until all chrs done
	DEC	C				; inner loop
	JR	NZ,Clear_l			; loop if not all done
	CALL	Set_cursor			; set cursor position (BC will be 0000)
;	LD	A,(DispMode)		; get mode
;	CALL	LCD_mode			; set display mode
	RET					; done

; set cursor address. 16 bit value held in BC. no registers destroyed

Set_cursor:
	PUSH	AF				; save AF
	LD	A,#0Ah			; Set cursor low addr instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,C				; set low address
	OUT	(LCD_D),A			; set mode

	LD	A,#0Bh			; Set cursor high addr instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,B				; set high address
	OUT	(LCD_D),A			; set mode

	LD	(CursorAt),BC		; save cursor position
	POP	AF				; restore AF
	RET

; Set graph mode. This will set the width to eight and the display to 15
; chrs/line. If you want the display on when graph mode is set call this
; routine with A<>0 .

LCD_setgramode:
	OR	A				; set the falgs
	LD	A,#12h			; display off/master/graph mode
	JR	Z,GsetOff			; jump if display is to be off
	LD	A,#32h			; display on/master/graph mode
GsetOff:
	LD	(DispMode),A		; save mode
	CALL	LCD_mode			; set display mode

	LD	A,#01h			; set chr pitch instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,#07h			; chr width 8 bits/chr
	OUT	(LCD_D),A			; set mode

	LD	A,#02h			; set no. of chrs instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,#0Eh			; 15 chr mode
	OUT	(LCD_D),A			; set mode

	LD	BC,0000h			; reset start of display
	JP	Set_start			; set start address & return

; Set character mode. This will read the default values and use those for
; the chr width and height, it will also set the display to 32 chrs/line.
; If you want the display on when chr mode is set call this routine with
; A<>0 .

LCD_setchrmode:
	OR	A				; set the falgs
	LD	A,#10h			; display off/master/chr mode
	JR	Z,CsetOff			; jump if display is to be off
	LD	A,#30h			; display on/master/chr mode
CsetOff:
	LD	(DispMode),A		; save mode
	CALL	LCD_mode			; set display mode

	LD	A,#02h			; Set no. of chrs instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,#1Fh			; 32 chr mode
	OUT	(LCD_D),A			; set mode

	LD	A,(ChrWidt)			; get chr width
	INC	A				; +1 for routine

; Set the character width for the chr mode display. Width is in A, valid
; widths are 6,7 and 8 bits. After the width is set the new visible length
; and the new start offset are set.

LCD_chrw:
	SUB	#06h				; convert to 0,1,2
	RET	C				; exit if was less than 6
	CP	#03h				; compare with 9
	RET	NC				; exit if not less (range 6 to 8)
	ADD	A,A				; *2
	ADD	A,A				; *4
	LD	HL,DispTbl			; get start of display table
	LD	C,A				; copy offset
	LD	B,#00h			; clear high byte
	ADD	HL,BC				; add offset
	LD	A,(HL)			; get width byte
	LD	(ChrWidt),A			; save new width
	LD	B,A				; copy width
	LD	A,(ChrHigh)			; get to chr height
	OR	B				; or in width nibble
	LD	B,A				; save size byte
	LD	A,#01h			; Set chr pitch instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,B				; get size byte back
	OUT	(LCD_D),A			; set mode
						; now set the other values
	INC	HL				; point to visible line length
	LD	A,(HL)			; get byte
	LD	(LineLen),A			; save it
	INC	HL				; point to offset low byte
	LD	C,(HL)			; get low byte
	INC	HL				; point to offset high byte
	LD	B,(HL)			; get high byte

; set display start address. 16 bit value held in BC. no registers destroyed

Set_start:
	PUSH	AF				; save AF
	LD	A,#08h			; Set display start low addr instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,C				; set low byte
	OUT	(LCD_D),A			; set mode

	LD	A,#09h			; Set display start high addr instruction
	CALL	LCD_instr			; send display control instruction
	LD	A,B				; set high byte
	OUT	(LCD_D),A			; set mode

	LD	(StrtOff),BC		; save start offset
	POP	AF				; restore AF
	RET

; Write chr to display. This routine will emulate a TTY display with auto
; scroll H (ON/OFF) and auto scroll V (ON/OFF) and a virtual display size
; of 32H x 64V characters. (may change it to 64H x 32V!)

; Control codes include ..

;	0Ah	Line Feed
;	0Dh	Carrage Return
;	08h	Backspace
;	09h	Tab

LCD_print:
	PUSH	HL				; save HL
	LD	L,A				; save A for now
	LD	A,(DispMode)		; get mode
	AND	#02h				; check chr/graph mode
	JR	NZ,nextbit			; exit if graph mode

	LD	A,L				; get chr back
	AND	#7Fh				; clear bit 7
	CP	#20h				; compare with 20h
	JP	M,ctrl_chr			; jump if control character
	
						; normal chr so out to display
	LD	A,#0Ch			; write display data
	CALL	LCD_instr			; set instruction
	LD	A,L				; get chr back
	OUT	(LCD_D),A			; chr out to display
						; now increment the cursor position
	PUSH	BC				; save BC
	LD	BC,(CursorAt)		; get cursor position
	INC	BC				; increment it
	JR	SetExit			; set cursor and exit

nextbit:
	POP	HL				; restore HL
	RET

ctrl_chr:					; control characters get here....
	PUSH	BC				; save BC
	LD	BC,(CursorAt)		; get cursor position
	LD	A,L				; get chr back
	CP	#0Dh				; was it CR
	JR	Z,WasCR			; jump if so

	CP	#0Ah				; was it LF
	JR	Z,WasLF			; jump if so

	CP	#09h				; was it TAB
	JR	Z,WasTA			; jump if so

	CP	#08h				; was it Backspace
	JR	Z,WasBA			; jump if so
	JR	CtrlDone			; else exit
SetExit:
	CALL	Set_cursor			; set the cursor
CtrlDone:					; gets here if not valid ctrl chr
	POP	BC				; restore BC
	POP	HL				; restore HL
	RET

; cariage return does this routine ..

WasCR:
	LD	A,#E0h			; set newline mask (C0h for 64H display)
	AND	C				; clear cursor x bits
	LD	C,A				; save new low byte
	JR	SetExit			; set cursor and exit

; line feed gets here ..

WasLF:
	LD	HL,0020h			; set linefeed offset (0040h for 64H display)
	ADD	HL,BC				; scroll to next line
	LD	C,L				; save new low byte
	LD	B,H				; save new high byte
	JR	SetExit			; set cursor and exit

; tab gets here ..

WasTA:
	LD	A,C				; get low byte
	AND	#F8h				; mask out TAB bits
	ADD	A,#08h			; add TAB step
	LD	C,A				; save low byte
	JR	NC,SetExit			; if no carry set cursor and exit
	INC	B				; else increment high byte
	JR	SetExit			; set cursor and exit

; backspace gets here

WasBA:
	DEC	BC				; step back one
	JR	SetExit			; set cursor and exit

; waits for no keys pressed then for a valid keypress. preserves the registers.

Wait_key:
	PUSH	HL				; preserve HL
	PUSH	DE				; preserve DE
	PUSH	BC				; preserve BC
wait_nloop:
	CALL	KEY_scan			; get a key
	JR	C,wait_nloop		; loop if invalid
	JR	NZ,wait_nloop		; wait until all keys released
wait_loop:
	CALL	KEY_scan			; get a key
	JR	C,wait_loop			; loop if invalid

	JR	Z,wait_loop			; wait until key pressed

	POP	BC				; restore BC
	POP	DE				; restore DE
	POP	HL				; restore HL
	RET

; scan the key matrix and return a key value. returns 00h if no key pressed
; and returns carry set if invalid (two keys or more) result.

KEY_scan:
	LD	B,#06h			; 6 rows to do
	LD	D,#01h			; set first bit
scan_loop:
	LD	A,D				; get scan row
	CALL	Key_byte			; out A (02h) in result (00h)
	JR	NZ,key_down			; find which (col) if not 0
	SLA	D				; set bit for next (row)
	DJNZ	scan_loop			; decrement count & loop for next
	XOR	A				; clear A and carry (no key)
	RET

; detected a bit set, so we got here ..

key_down:
	LD	E,A				; save returned scan (col) byte
	LD	A,D				; get scan (row) byte
	CPL					; invert bits
	AND	#3Fh				; clear top two bits
	CALL	Key_byte			; out A (02h) in result (00h)
	JR	NZ,two_key			; jump if more than 1 key pressed

	LD	A,#06h			; to convert row 6-1 to 0-5
	SUB	B				; subtract row # (A now contains row 0-5)
	LD	C,A				; save (row) result
	LD	A,E				; restore returned scan (col) byte
	CALL	Get_one			; scan (col) for first bit = 1
	JR	C,two_key			; jump if more than 1 key pressed

	SLA	A				; result *2
	LD	D,A				; copy it
	SLA	A				; result *4
	ADD	A,D				; result *6
	ADD	A,C				; add (row) #
	LD	HL,(key_ptr)		; get key table address pointer

	LD	C,A				; copy it
	LD	B,#00h			; clear high byte
	ADD	HL,BC				; add to pointer (also clears carry)
	LD	A,(HL)			; get byte from table
	RET					;

two_key:
	XOR	A				; clear result
	SCF					; set carry
	RET					; done

; out A (02h) in result (00h)

Key_byte:
	OUT	(KBD_W),A			; out to port
	PUSH	HL				; wait a while ..
	POP	HL				; ..
	PUSH	HL				; ..
	POP	HL				; ..
	IN	A,(KBD_R)			; in from key matrix
	AND	#3Fh				; mask byte
	RET					;

; search A (bits 0-5) for a 1. return position of first bit=1 in A.
; returns carry set if more than 1 bit=1

Get_one:
	PUSH	BC				; save BC
	PUSH	DE				; save DE
	LD	BC,0600h			; clear bit=1 count & bit #
	LD	D,B				; clear result
bit_loop
	RRA					; bit 0 to carry
	JR	NC,not_one			; jump if was 0

	LD	D,B				; else copy bit # to result
	INC	C				; increment bit=1 count
not_one
	DJNZ	bit_loop			; loop if not all done

	LD	A,#06h			; to convert 6-1 to 0-5
	SUB	D				; subtract bit #
	LD	D,A				; copy it
	LD	A,#01h			; max o.k. bit count
	CP	C				; compare with count (set carry if c>1)
	LD	A,D				; copy result to A
	POP	DE				; restore DE
	POP	BC				; restore BC
	RET					;

Key_tab:					; key matrix
	DB	80h,'A','B',FFh,'1',7Fh	; 80h is shift key, FFh is @ key
	DB	'C','D','E','F','2','0'
	DB	'G','H','I','J','3',01h	; 01h is NO key
	DB	'K','L','M','N',0Dh,06h	; 06h is YES key
	DB	'7','8','9',D0h,00h,00h	; D0h is DEPT key
	DB	'4','5','6',14h,00h,00h	; 14h is END key

Key_hex:					; key matrix (hex functions)
	DB	80h,0Ah,0Bh,1Fh,01h,7Fh	; 80h is shift key, 1Fh is @ key
	DB	0Ch,0Dh,0Eh,0Fh,02h,00h
	DB	'G','H','I','J',03h,11h	; 11h is NO key
	DB	'K','L','M','N',1Dh,16h	; 1Dh IS enter KEY, 16h is YES key
	DB	07h,08h,09h,D0h,FFh,FFh	; D0h is DEPT key
	DB	04h,05h,06h,14h,FFh,FFh	; 14h is END key

Set_def:
	DW	Key_tab			; default keymap
	DB	00h				; default port address
	DW	0000h				; mon_addr default
	DB	00h				; mon_mode default (address mode)
LCD_def:
	DB	08h				; TabSize default - display TAB step size
	DW	0000h				; CursorAt default - cursor address in display
	DB	10h				; DispMode default - display mode byte
						; (off/master/noblink/nocsr/cgr/intcg)
	DB	70h				; ChrHigh default - current character height
						; (8 default)
	DB	05h				; ChrWidt default - current character width
						; (6 default)
	DB	20h				; LineLen default - current visible line length
						; (32 default)
	DW	FFF4h				; StrtOff default - default display start offset
	DW	0000h				; DispOff default - offset for display
	DB	90h				; DispCon default - display contrast
						; DispTbl defaults
	DB	05h,14h,F4h,FFh		; 6 bits/chr, 20 chrs vis, -0Ch offset
	DB	06h,11h,F1h,FFh		; 7 bits/chr, 17 chrs vis, -0Fh offset
	DB	07h,0Fh,EFh,FFh		; 8 bits/chr, 15 chrs vis, -11h offset
	DB	00h				; default bank page
	DB	00h				; default bank page
Defend:
Tst1:
	DB	0Ah
	DB	'Testing RTC ..'
Tst1End:
MPassed:
	DB	0Dh,0Ah
	DB	'Passed'
MPaend:
MFailed:
	DB	0Dh,0Ah
	DB	'Failed'
MFaend:
Tst2:
	DB	0Dh,0Ah
	DB	0Ah
	DB	'Press any key >>'
Tst2End:
Mon:
	DB	0Ah
	DB	'Mem page is - ',0Dh,0Ah
	DB	'Address  is - ',0Dh,0Ah
	DB	'Content  is - ',0Dh,0Ah
	DB	0Ah
	DB	'Area is '
MonEnd:
RamM
	DB	'RAM    '
RamEnd
NRamM
	DB	'not RAM'
NRamEnd
Msg:
	DB	0Ah
	DB	'*  Micronic  1000  *',0Dh,0Ah
	DB	'*   LCD  monitor   *',0Dh,0Ah
	DB	0Ah
	DB	'*   software  by   *',0Dh,0Ah
	DB	'*  L.Davison 1998  *',0Dh,0Ah
	DB	0Ah
	DB	'Press any key >>'
MsgEnd:
HelpM:
	DB	'Insert - addr mode',0Dh,0Ah
	DB	'Enter  - data mode',0Dh,0Ah
	DB	'G - Go from addr',0Dh,0Ah
	DB	'J - Port monitor',0Dh,0Ah
	DB	'K - blocK dump',0Dh,0Ah
	DB	'L - set mem bank',0Dh,0Ah
	DB	'M - warm start',0Dh,0Ah
	DB	'Z - cold start   >>'
HelpEnd:
PageM:
	DB	'Set banked mem page',0Dh,0Ah
	DB	0Dh,0Ah
	DB	'Current page is',0Dh,0Ah
	DB	0Dh,0Ah
	DB	'New page is ...',0Dh,0Ah
	DB	0Dh,0Ah
	DB	'Enter - Set new bank',0Dh,0Ah
	DB	'End   - Exit'
PageEnd:

; routines stolen from the original ROM

; set clock for BCD mode, 12hr mode & 64Hz timebase

LAB_20D9
	LD	A,#40h		; set PIE, BCD & 12hr mode
	LD	B,#0Bh		; set RTC address 0Bh
	CALL	LAB_22DB		; write A to RTC port (B)
	LD	A,#26h		; set divider mode 2 & rate mode Ah
	LD	B,#0Ah		; set RTC address 0Ah
	CALL	LAB_22DB		; write A to RTC port (B)
	RET				;

; write A to RTC port (B)

LAB_22DB
	LD	C,#RTC_A		; set RTC address port
	OUT	(C),B			; set RTC address
	OUT	(RTC_D),A		; A out to RTC
	RET				;

; read A from RTC port (B)

LAB_22E2
	LD	C,#RTC_A		; set RTC address port
	OUT	(C),B			; set RTC address
	IN	A,(RTC_D)		; A in from RTC
	RET				;

; enable all IRQs & setup interrupt table

LAB_22E9
	LD	A,#1Fh		; set IRQ hardware mask ??
LAB_22EB
	DI				; disable interrupts
	IM	1			; set mode 1
	CPL				; complement A (1=enable)
;	LD	(LAB_F784),A	; set Port_04 (interrupt mask) buffer
	OUT	(Port_04),A		; out to Port_04
	LD	A,#01h		; set value
;	LD	(LAB_FFA8),A	; save interrupts setup ?? flag
	LD	HL,LAB_2352		; source address
	LD	DE,LAB_FD84		; destiantion address
	LD	BC,0013h		; 0013h (19d) bytes to copy
	LDIR				; copy block
	EI				; enable interrupts
	RET				;

; enable RTC IRQ & setup interrupt table

LAB_2306
	LD	A,#02h		; set RTC IRQ hardware unmask
	JR	LAB_22EB		; IRQ hardware unmask & copy what interrupt table ??

; what interrupt table, copied to LAB_FD84

LAB_2352
	DB	01
	DW	LAB_18F0		; scan keyboard matrix ??
	DB	02
	DW	LAB_2206		; check RTC interrupt flags
	DB	04
	DW	LAB_31B6		; IR port interrupt routine ??
	DB	08
	DW	LAB_2365		; main battery warning ??
	DB	10
	DW	LAB_2365		; backup battery warning ??
	DB	00
	DW	LAB_0000		; this never happens !
	DB	80			; end marker

; just for now ...

LAB_0000:
LAB_18F0:
LAB_2206:
LAB_2365:
LAB_31B6:
	RET				;

; test clock

LAB_2828
	LD	DE,LAB_2877		; IRQ vector to LAB_2877
	CALL	LAB_2893		; save JP DE @ IRQ routine address
	LD	HL,0082h		; set test count
	LD	(LAB_FDA8),HL	; save test count
;	LD	D,#26h		;2834:	1626
	CALL	LAB_20D9		; set clock for BCD mode, 12hr mode & 64Hz timebase
	CALL	LAB_288A		; clear main and RTC interrupt flags
	CALL	LAB_2306		; enable RTC IRQ & setup interrupt table

	LD	BC,0000h		; inner loop delay count
	LD	D,#02h		; outer loop delay count

LAB_2844
	INC	BC			; [6]	increment count
	LD	A,B			; [4]	get high byte
	OR	C			; [4]	or with low byte
	JP	NZ,LAB_2844		; [12]loop until BC=0
					; [7]

	DEC	D			; [4]	decrement loop count
	JP	NZ,LAB_2844		; [12]loop until D=0
					; [7] total of 4194333 cycles

; gets here after loop timeout or (LAB_FDA8) RTC interrupts

LAB_284E
	DI				; interrupts off
	LD	A,#FFh		; set mask byte
	OUT	(Port_04),A		; disable all interrupts
	LD	A,#02h		; compare byte
	CP	D			; compare with outer loop count
	JR	NZ,LAB_286C		; jump if <> 02h (way too slow)

	LD	DE,4502h		; set lower limit
	CALL	LAB_2872		; subtract BC from DE, sets carry flag
	JR	NC,LAB_286C		; failed, clock running too fast

	LD	DE,4C46h		; set upper limit
	CALL	LAB_2872		; subtract BC from DE, sets carry flag
	JR	C,LAB_286C		; failed, clock running too slowly

	LD	A,#00h		; else set passed
	JR	LAB_286E		; set flags & exit

LAB_286C
	LD	A,#FFh		; set failed
LAB_286E
;	LD	(LAB_FDB5),A	; save clock test status flags
	RET				;

; subtract BC from DE, sets carry flag

LAB_2872
	LD	A,E			; copy E
	SUB	C			; subtract C
	LD	A,D			; copy D
	SBC	A,B			; subtract B
	RET				;

; clock test IRQ routine

LAB_2877
	DI				; disable interrupts
	LD	A,(LAB_FDA8)	; get test interrupt count
	DEC	A			; decrement it
	LD	(LAB_FDA8),A	; save test interrupt count
	JR	Z,LAB_2886		; exit if count =0

	CALL	LAB_288A		; clear main and RTC interrupt flags
	EI				; enable interrupts
	RET				;

					; done test clock interrupt count
LAB_2886
	POP	IX			; dump interrupt return address
	JR	LAB_284E		; go analyse test result

; clear main and RTC interrupt flags

LAB_288A
	IN	A,(Port_05)		; clear interrupt flags
	LD	A,#0Ch		; set RTC address port 0Ch
	OUT	(RTC_A),A		; set RTC address
	IN	A,(RTC_D)		; get RTC data (get & clear interrupt flags)
	RET				;

; save JP DE @ IRQ routine address

LAB_2893
	LD	HL,LAB_F5F3		; point to IRQ routine address
	LD	A,#C3h		; HEX for JP
	LD	(HL),A		; save it
	INC	HL			; point to next byte
	LD	(HL),E		; save low byte of JP address
	INC	HL			; point to next byte
	LD	(HL),D		; save high byte of JP address
	RET				;

; these need to be in RAM. as I know 8800h on is always RAM I'll use that

;	ORG	8800h				; moved to RAM allready

key_ptr:
	DS	02h				; pointer to key table
port_adr:
	DS	01h				; current port address
mon_addr:
	DS	02h				; current monitor address
mon_mode:
	DS	01h				; current monitor mode
TabSize:
	DS	01h				; display TAB step size
CursorAt:
	DS	02h				; cursor address in display
DispMode:
	DS	01h				; display mode byte
ChrHigh:
	DS	01h				; current character height (8 default)
ChrWidt:
	DS	01h				; current character width (8 default)
LineLen:
	DS	01h				; current visible line length (32 default)
StrtOff:
	DS	02h				; default display start offset
DispOff:
	DS	02h				; offset for display
DispCon:
	DS	01h				; display contrast
DispTbl:
	DS	0Ch				; display parameters table
bank_p:
	DS	01h				; memory bank page
bank_b:
	DS	01h				; new memory bank page
LAB_FD84:
	DS	13h				; interrupt table
LAB_FDA8:
	DS	02h				; interrupt test count word

	END